///////////////////////////////////////////////////////////
//                                                       //
//                 File: scoring.cpp                     //
//               Author: Scott Arrington                 //
//                       Fall '97 EE499                  //
//                       UK IEEE Student Branch          //
//                                                       //
// Implementation of the class ScoringEngine for the     //
// 1999 IEEE SouthEast CON Hardware Design Competition   //
//                                                       //
//                                                       //
// This is the class that receives the data from the     //
// port driver and computes the score accordingly.       //
//                                                       //
///////////////////////////////////////////////////////////

#include "scoring.h"
#include "events.h"

extern TeamList _Global;
Port Driver;


// public methods

// constructor
ScoringEngine::ScoringEngine(void)
	{
	Reset();
	}



//////////////////////////////////////
//  reset the whole thing.
//
void ScoringEngine::Reset(void)
	{
	resetTeamIds();
	resetEventRecord();
	_bitmap[TRACKA] = _start;
	_bitmap[TRACKB] = _start;
	_cheat[TRACKA]  = FALSE;
        _cheat[TRACKB]  = FALSE;
	}

// reset internal ID numbers
void ScoringEngine::resetTeamIds(void)
	{
	teamAid = TEAM_UNSET ;
	teamBid = TEAM_UNSET ;
	}

// clear the event record for the start of a new competition
void ScoringEngine:: resetEventRecord(void)
	{
	// there are only 12 events!!
	for (int i=0 ; i<2 ; i++)
		{
		for (int j=0;  j<12 ; j++)
			_event_record[i][j] = FALSE;
		}
	}


/////////////////////////////////////////////////////////
//  do all the necessary stuff to get the cars rolling
//
//     for the competition, these two are called in order:
//	   first reset the track to watch for cheaters
//	   then sound call startCompetition to sound the buzzer 


//  this is called by the track window to tell the software to 
//  start reading the track events.
void ScoringEngine::resetTrack(void)
{

	Driver.reset();

}


// this is the finction to be called by the main window's "Start" button;
// it checks for cheaters (those cars that moved before the buzzer),
// then sounds the buzzer to tell the cards it's time to go.

void ScoringEngine::startCompetition(void)
	{
	int event;

	// note that the getEvent() call clears the event after reading,
	// so that no points are awarded for anything that happened
	// *before* the buzzer.  the event record is set to the proper
	// event for the purposes of the bitmap update. (i think)  

	// check for cheaters
	event = Driver.getEvent(TRACKA);
        if (event)
		{
		_cheat[TRACKA] = TRUE;
		_event_record[TRACKA][event] = TRUE;
		_bitmap[TRACKA] = _cheated;
		}

	event = Driver.getEvent(TRACKB);
        if (event)
		{
		_cheat[TRACKB]=TRUE;
		_event_record[TRACKB][event] = TRUE;
		_bitmap[TRACKB] = _cheated;
		}

        // tell the track to fire the starting pistol
	Driver.buzzer();

        // that's it
        }




/////////////////////////////////////////
// Set which team is on which track
//
void ScoringEngine::setTeamIds(int teamA, int teamB)
	{
	teamAid = teamA ;
	teamBid = teamB ;
	}

//////////////////////////////////////////////////////////////////
// is time for the main window to change the picture of the track?
// this occurs if the vehicle has completed a section; return true for
// the proper events, and false for the others.
//
int ScoringEngine::updateBitmap(int track)
	{
	return(_bitmap[track]);
	}



////////////////////////////////////////////////
//	the *other* big do-all function
//	   (calls private functions)
//
void ScoringEngine::updateScore(void)
{
  _Global.SetScore(teamAid,computeScore(TRACKA));
  _Global.SetScore(teamBid,computeScore(TRACKB));
}





//////////////////
// private stuff

////////////////////////////////////////////////////////////
// this is the routine that actually calculates the score...
// any changes to the scoring are done here!
//
//    NOTE:  the internal representation of the score (in teamlist)
//           is *twice* the actual score!  this is to prevent losses
//	     due to integer division, since we can't afford scoretype
//	     to be float in this 16-bit environment.
//
scoretype ScoringEngine::computeScore(int trackno)
	{
	scoretype score;
	int event;
	
	// score = current score + new points for last event:
	// first, get the current score:
	
	if (trackno)
		score = 2*_Global.GetScore(teamBid);
	else
		score = 2*_Global.GetScore(teamAid);

	// account for cheating:  if they cheated, their score in
	// the teamlist will be halved.  normalize for the math here,
	// then return score/2 at the end.

	if (_cheat[trackno])
        	score = score * 2;

	// now find out what has happened, and score accordingly:
	
	event = Driver.getEvent(trackno);

	// if something has happened and has not been seen:
	//	record the event in event record,
	//      and increment the score accordingly

	if ((event)&&(!_event_record[trackno][event]))
		{
		_event_record[trackno][event] = TRUE;
		switch( event )
			{
			case START:
				score += StartPOINTS ;
				_bitmap[trackno] = _city ;
                                //_bitmap[trackno] = _start;
				break;

			case ATM:
				score += AtmPOINTS ;
				_bitmap[trackno] = _city ;
				break;

			case STOPLIGHT:
				score += StoplightPOINTS ;
				_bitmap[trackno] = _city ;
				break;

			case BOUND1:
				score += Bound1POINTS ;
				_bitmap[trackno] = _auction ;
				break;

			case HORSE:
				score += HorsePOINTS ;
				_bitmap[trackno] = _auction ;
				break;

			case PAVILION:
				score += PavilionPOINTS ;
				_bitmap[trackno] = _auction ;
				break;

			case BOUND2:
				score += Bound2POINTS ;
				_bitmap[trackno] = _start;
				break;

			case MOUNTAIN:
				score += MountainPOINTS ;
				_bitmap[trackno] = _mountain ;
				break;

			case DOOR:
				score += DoorPOINTS ; 
				_bitmap[trackno] = _mine ;
				break;

			case MINE_EXIT:
				score += MineexitPOINTS ;
				_bitmap[trackno] = _mine ;
				break;

			case END:
				score += EndPOINTS ;
				_bitmap[trackno] = _end ;
				break;


			case NO_EVENT:
				break;
				// do nothing here
			}
		}
	
	//  again, account for any cheating (see above)
	if(_cheat[trackno])
        	score = score / 2;

        // and finally...
	return(score);

}
