////////////////////////////////////////////////////////////////
//     Scoring.h   --- Definition of class ScoringEngine      //
//                                                            //
// receives data from the port driver and computes the score  //
// accordingly.                                               //
//                                                            //
//     Author:   Scott Arrington                              //       
//               UK IEEE and EE499                            //
//       Date:   December 1997                                //
//                                                            //
//      This file is part of the scoring software for the     //
//         UK IEEE SouthEastCON'99 Hardware Competition       //
//                                                            //
////////////////////////////////////////////////////////////////
#ifndef _SCORING_H
#define _SCORING_H

#include "portdrv.h"
#include "teamlist.h"

#define TEAM_UNSET 40

/*
 *  define the names for the numbered bitmaps
 */

#define      _start	1
#define       _city	2
#define    _auction	3
#define   _mountain	4
#define       _mine	5
#define        _end	6
#define    _cheated	7


/*  These allocate the points for each event!  
 *
 *  ******  changes in scoring should  be made here !!! **********
 *
 *     note that the score is kept internally twice as large
 *     to avoid truncation in integer division;  this means
 *     that the defined value is two times the real value
 */

#define      StartPOINTS    0      //  0 x 2
#define        AtmPOINTS   40	   // 20 x 2
#define  StoplightPOINTS   60      // 30 x 2
#define     Bound1POINTS   10      //  5 x 2
#define      HorsePOINTS   20      // 10 x 2
#define   PavilionPOINTS   80      // 40 x 2
#define     Bound2POINTS   10      //  5 x 2
#define   MountainPOINTS  100      // 50 x 2
#define       DoorPOINTS   50      // 25 x 2
#define   MineexitPOINTS   50      // 25 x 2
#define        EndPOINTS   20      // 10 x 2
				   //      this last is 5+5 -- the loser
				   //      points of both the mine and 
				   //      the mountain... see the rules


////////////////////////////////////////////////////////////////

class ScoringEngine 
{
   public:
	ScoringEngine(void);		// constructor

	void Reset(void);		// reset the scoring engine
					// for the next competition

	void startCompetition(void);	// look for cheaters and sound
        				// the buzzer

	void setTeamIds(int teamA, int teamB);
		// call setTeamIds to tell the scoring engine 
		// which team is on which track: 
		//	teamA is on track A
		//	teamB is on track B

	int updateBitmap(int track);	// which picture to display?

	BOOL isEnd(int track)		// is it time to stop the timer?
		 { return( _event_record[track][END] );}

	void updateScore(void);		// call to check the track 
					// and update the scores accordingly
	void resetTrack(void);


  private:

	int  teamAid, teamBid;		// which team is where
	void resetTeamIds(void);	// reset internal ID numbers

	BOOL _event_record[2][12];	// record of what has happened
	void resetEventRecord(void);	// reset the record

	int  _bitmap[2];		// which bitmap to display
	BOOL _cheat[2];			// did they cheat?


	scoretype computeScore(int trackno);
					// this is where the action is:
                                        // reading the port data, 
					// calculating scores,
					// and logging events

};

#endif  // _SCORING_H
