///////////////////////////////////////////////////////////
//                                                       //
//                 File: scoring.cpp                     //
//               Author: Keith Loiselle                  //
//                       Fall '97 EE499                  //
//                       UK IEEE Student Branch          //
//                                                       //
// Implementation of the class Port for the              //
// 1999 IEEE SouthEast CON Hardware Design Competition   //
//                                                       //
//                                                       //
// This is the class that communicates with the parallel //
// port and the attached hardware, and interprets the    //
// signals it receives.                                  //
//                                                       //
///////////////////////////////////////////////////////////

#include "portdrv.h"


//  these are the values that correspond to the actual
//  hardware setup.  see the schematics before thinking
//  about messing with them.
#define RESET_BIT 2
#define BUZZER_BIT 4
#define SELECT_BIT 0

////////////////////////////////////////////////
//  constructor -- initialize port for first use
//
Port::Port(int baseAddr)
{
	dataAddr = baseAddr;
	statusAddr = baseAddr + 1;
	controlAddr = baseAddr + 2;

	data = 0xC0;
	outportb(dataAddr, data);
	outportb(controlAddr, 0x00);

	for (int board = 0; board <= 1; board++)
	{
		lastEvent[board] = 0;
		curSection[board] = 0;
		reset();
		setBuzzer(board, 0);
	}
}


//////////////////////////////////////////////////////////
//  what has happened on the track?  this function returns
//  the event code (as defined in events.h) for the last
//  signal to be seen at the parallel port, and zero if
//  that event has already been seen.
//
int Port::getEvent(int boardNum)
{
	select(boardNum, curSection[boardNum]);
//	delay(SELECT_DELAY);
	int status = inportb(statusAddr) ^ 0x8C;
	int event = (((~status) >> 5) & 0x07) + 7*curSection[boardNum];
//	delay(SELECT_DELAY);

	if (event < lastEvent[boardNum])
		return -1;
	else if (event == lastEvent[boardNum])
		return 0;
	else
	{
		if (event == 7)
			curSection[boardNum] = 1;

		lastEvent[boardNum] = event;
		return event;
	}
}


/////////////////////////////////////////////////////////////
//   this method tells the driver to start counting from zero,
//   and sets the value of the "reset" line.
//
void Port::setReset(int boardNum, int value)
{
	setDataBit(RESET_BIT + boardNum, value);

	if (value)
	{
		lastEvent[boardNum] = 0;
		curSection[boardNum] = 0;
	}
}


/////////////////////////////////////////////////
//  sets the value of the buzzer line.
//
void Port::setBuzzer(int boardNum, int value)
{
	setDataBit(BUZZER_BIT + boardNum, value);
}

///////////////////////////////////////////////////////////////
// low-level function to set a parallel port line to a given value.
//
void Port::setDataBit(int bit, int value)
{
	if (value)
		data = data | (0x01 << bit);
	else
		data = data & (~(0x01 << bit));

	outportb(dataAddr, data);
}


////////////////////////////////////////////////////////////
//  send a pulse down the "buzzer" line of both tracks.
//  this pulse is supposed to trigger the buzzer, which
//  marks the start of the competition. 
//
//  the pulse width is determined by the amount of delay.
//  this is likelyprocessor-dependent; adjust BUZZER_DELAY
//  in portdrv.h to change it.
//
void Port::buzzer()
{
	setBuzzer(TRACKA, 1);
	setBuzzer(TRACKB, 1);

//	delay(BUZZER_DELAY);
// THIS IS ANOTHER KLUDGE
for (int j = 0 ; j < BUZZER_DELAY * 10000 ; j++ ) ;

	setBuzzer(TRACKA, 0);
	setBuzzer(TRACKB, 0);
}



////////////////////////////////////////////////////////
//  send a pulse down the "reset" line.  this tells the
//  track to re-initialize everything to the ready state.
//
void Port::reset()
{
	setReset(TRACKA, 1);
	setReset(TRACKB, 1);

//	delay(RESET_DELAY);
// THIS IS A KLUDGE
for (int j = 0 ; j < RESET_DELAY * 10000 ; j++ );


	setReset(TRACKA, 0);
	setReset(TRACKB, 0);
}


/////////////////////////////////////////////////////////////
//  allows the driver to choose which track it wants to read.
//
void Port::select(int boardNum, int section)
{
	setDataBit(SELECT_BIT, section);
	setDataBit(SELECT_BIT+1, boardNum);
}
