////////////////////////////////////////////////////////////////
//     portdrv.h   --- Definition of class Port               //
//                                                            //
// This class performs the actual software interface with     //
// the parallel port hardware of the track.  It can send      //
// the signal to sound the buzzer, and interprets the         //
// events received from either track.                         //
//                                                            //
//     Author:   Keith Loiselle                               //       
//               UK IEEE and EE499                            //
//       Date:   December 1997                                //
//                                                            //
//      This file is part of the scoring software for the     //
//         UK IEEE SouthEastCON'99 Hardware Competition       //
//                                                            //
////////////////////////////////////////////////////////////////


#ifndef _PORT_H
#define _PORT_H

#include <dos.h>
#include <windows.h>

// include the file containing all the event code definitions
#include "events.h"

// give names to the tracks
#define TRACKA 0
#define TRACKB 1

// define base numbers for the counter delays; these will be
// multiplied by 10000.  (the delays are for() loops; the amount
// of delay is processor-dependent.
#define SELECT_DELAY 0
#define BUZZER_DELAY 100
#define RESET_DELAY 100


_CLASSDEF(Port)

class Port
{
    public:
	Port(int baseAddr = 0x0378);	// constructor;  if you're
					// not using LPT1, change this
                                        // address

	int getEvent(int boardNum);	// return the code of the last
					// event; if the event has been
					// seen, return 0.  see events.h

	void setReset(int boardNum, int value);	// explicit line set
	void setBuzzer(int boardNum, int value);// explicit line set
	void buzzer();				// pulse the buzzer line
	void reset();				// pulse the reset line

    private:
	void select(int boardNum, int section);
					// chooses track/section to read
	void setDataBit(int bit, int value);	// low-level port write       
	int data;				// current value if data line
	int dataAddr;				// base address
	int statusAddr;				// base address + 1
	int controlAddr;			// base address + 2
	int lastEvent[2];			// last event that happened
	int curSection[2];			// drives the multiplexer
};


#endif   /*  _PORT_H */