///////////////////////////////////////////////////////////
//                                                       //
//                 File: teamlist.cpp                    //
//              Authors: Earl Fugate and                 //
//                       Scott Arrington                 //
//                       Fall '97 EE499                  //
//                       UK IEEE Student Branch          //
//                                                       //
// Declaration of the class TeamList for the             //
// 1999 IEEE SouthEast CON Hardware Design Competition   //
//                                                       //
//                                                       //
// This is the class that stores and manipulates the     //
// data pertaining to the contestant teams.  It works    //
// very closely with the ScoringEngine and the track     //
// window...                                             //
// Because teams that start early get half points, and   //
// points are awarded in multiples of five, it is        //
// possible to have fractional parts of the score.       //
// For this reason, GetScore() returns a float.          //
//                                                       //
///////////////////////////////////////////////////////////

#ifndef _TEAMLIST_H
#define _TEAMLIST_H

typedef int scoretype;

#include <string.h>

_CLASSDEF(TeamList)

class TeamList
{
   public:
	TeamList();			// constructor
	void AddtoList(char*);		// add a team, supplying a pointer to the name
	char* GetfromList(int);		// get the name of a team 
	void RemovefromList(int);	// remove a team from the list
	void DeleteListFrom(void);	// delete the list

	float GetScore(int team) { return ( _score[team]/2.0 ) ; }
		// return the score of the team with index number "team"

	void SetScore(int team,scoretype score) { _score[team] = score; }
        	// set the score for the desired team to the suppied value

	int GetTime(int team) { return _time[team]; }
        	// return the time for team "team" 
	void SetTime(int team,int time) { _time[team] = time; }
        	// set the time for team "team" to "time"
		
	void SortScores(void);		// sort the team list by scores;
        				// the times break ties

	void RemovefromScoreList(int);	// cleanup functions...
	void RemovefromTimeList(int);


   private:
	int  _index;	// the individual teams are accessed via an
			// integer index.  this fact is hidden from the
			// end user but is vital to all accessors.
	char _holder[41][40];   // the list of names; the last is for NONE
	int  _time[41];		// the list of times, used to break ties
	scoretype _score[41];   // the list of scores;
				// NOTE: these are stored as *twice*
				// their actual value to aviod loss due
				// to truncation in integer division
                         
};


#endif   /*  _TEAMLIST_H */
