///////////////////////////////////////////////////////////
//                                                       //
//                 File: track.cpp                       //
//               Author: Earl Fugate                     //
//              Hackers: Keith and Scott                 //
//                       Fall '97 EE499                  //
//                       UK IEEE Student Branch          //
//                                                       //
// Implementation of the track window for                //
// 1999 IEEE SouthEast CON Hardware Design Competition   //
// The competition takes place here.                     //
//                                                       //
// Built on Borland's Object Windows Library ---  much   //
// of this code is dealing with that stuff.              //
//                                                       //
///////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include "teamlist.h"
#include "track.h"
#include "select.h"
#include "scoring.h"
#include "events.h"


extern TeamList _Global;           //  declared globally in ieee.cpp
extern int _NumberofTeams;
extern int _ScreenWidth;
extern EraseBackground;

int TrackABmp, TrackBBmp, XPlace, OldRC;
int TrackATeam, TrackBTeam, OldTeamA, OldTeamB;
BOOL Initialize, SelectedTeams;
BOOL StartTimer;
BOOL TimerCountA, TimerCountB;
BOOL ready;

char Space[45] = "                                             ";
struct time t;
int basesec, newAsec, basemin, newAmin, newBsec, newBmin;
ScoringEngine Scoring;

/*
char events[12][25]={
                       "Nothing new",
                       "Start signal",
                       "ATM button pushed",
                       "Stoplight passed",
                       "Left city section",
                       "Horse picked up",
                       "Horse dropped off",
                       "Left Bluegrass section",
                       "Mountaintop reached",
                       "Mine Door Opened",
                       "Left the Mine",
                       "End signal"
                       };
*/


////////////////////////////////////////////////////////
// constructor, performing more initialization before //
// 9 am than most constructors do all day.            //
////////////////////////////////////////////////////////
Track::Track( PTWindowsObject AParent, LPSTR ATitle ) :
	 TBaseDemoWindow(AParent, ATitle)
{
  

  int width = 340;
  int heigt = 440;      // old h = 240;
  int starx = 300;
  int stary = 0;	//old y = 200;



  Iconized = FALSE;

  if (_ScreenWidth == 800)
	{
	width = width * 1.25;
	heigt = heigt * 1.25;
	starx = starx * 1.25;
	stary = stary * 1.25;
        }

  Attr.W = width;
  Attr.H = heigt;
  Attr.X = starx;
  Attr.Y = stary;

  // creates and lays out the buttons
  ButtonStart	= new TButton(this, ID_BUTTONSTART, "Start", 10, 80, 100, 30, FALSE);
  ButtonStop  	= new TButton(this, ID_BUTTONSTOP,  "Stop" , 10, 115, 100, 30, FALSE);
  ButtonTeam	= new TButton(this, ID_BUTTONTEAMS, "Team Select", 10, 10, 100, 30, FALSE);
  ButtonReset	= new TButton(this, ID_BUTTONRESET, "Reset", 10, 45, 100, 30, FALSE);

  TrackABmp = TrackBBmp = 1;
  TrackATeam = TrackBTeam = OldTeamA = OldTeamB = 40;

  Initialize = TRUE;
  StartTimer = FALSE;
  newAsec = newBsec = newAmin = newBmin = 0;
  TimerCountA = TimerCountB = FALSE;
  SelectedTeams = FALSE;
  ready=FALSE;


}


////////////////////////////////////////////////
//  bitmap selection occurs here...
//
void Track::GetWindowClass( WNDCLASS& WndClass )
{

  TBaseDemoWindow::GetWindowClass( WndClass );
  WndClass.hbrBackground = CreateSolidBrush(0x0);
  WndClass.hIcon = LoadIcon( GetApplication()->hInstance, "ieee");
  WndClass.style = CS_NOCLOSE;

  Blank   = LoadBitmap(GetApplication()->hInstance, "blank");
  Over[_start]    = LoadBitmap(GetApplication()->hInstance, "track0");
  Over[_city]     = LoadBitmap(GetApplication()->hInstance, "track1");
  Over[_auction]  = LoadBitmap(GetApplication()->hInstance, "track2");
  Over[_mountain] = LoadBitmap(GetApplication()->hInstance, "track3");
  Over[_mine]     = LoadBitmap(GetApplication()->hInstance, "track4");
  Over[_end]      = LoadBitmap(GetApplication()->hInstance, "track5");
  Over[_cheated]  = LoadBitmap(GetApplication()->hInstance, "cheat");
}


/////////////////////////////////////////////
//   actually paints and re-paints the window
//   checks to see if anything has changed,
//   and repaints that which is out of date.
//
void Track::Paint( HDC , PAINTSTRUCT& )
{

  RECT rc;
  GetClientRect(HWindow, &rc);

  if (((rc.right <= 340) || ((rc.right <= 425) && (_ScreenWidth == 800))) && Initialize)
	{
	XPlace = 120;
	Initialize = FALSE;
        InvalidateRect(HWindow, NULL, TRUE);
        OldRC = rc.right;
        }

  if (OldRC != rc.right)
	{
	XPlace = ((rc.right - 150) / 2);
	InvalidateRect(HWindow, NULL, TRUE);
	OldRC = rc.right;
        Initialize = TRUE;
	}
	  
  HDC DC = GetDC(HWindow);


  DrawBMP( DC, XPlace, 30, Over[TrackABmp] );
  DrawBMP( DC, XPlace, 200, Over[TrackBBmp]);
  SetBkColor(DC, RGB(0, 0, 0));
  SetTextColor(DC, RGB(255, 255, 255));

  if(OldTeamA != TrackATeam)
	{
	TextOut(DC, XPlace, 130, Space, strlen(Space));
	}


  if(OldTeamB != TrackBTeam)
	{
	TextOut(DC, XPlace, 305, Space, strlen(Space));
	}

  TextOut(DC, XPlace, 10, "Track A", 7);
  TextOut(DC, XPlace, 180, "Track B", 7);
  TextOut(DC, XPlace, 130, _Global.GetfromList(TrackATeam),
		strlen(_Global.GetfromList(TrackATeam)));
  TextOut(DC, XPlace, 305, _Global.GetfromList(TrackBTeam),
	 	strlen(_Global.GetfromList(TrackBTeam)));


  ReleaseDC(HWindow, DC);
                
}


////////////////////////////////////////////////////////////
//   This is what happens when the Select button is pressed.
//   The aim of the button is to select the teams which will
//   compete on the two tracks.  These teams are tracked as
//   the index of the array in _Global; these numbers are
//   essential to the proper execution of the scoring code,
//   and thus this button is all-important.
//
void Track::ButtonSelectMsg(RTMessage)
{
  GetApplication()->MakeWindow(new SelectTeams(this, "Select Teams"));

}



/////////////////////////////////////////////////////////////////////
//  These instructions are executed upon pressing the "Reset" button.
//   This is step one in starting the competition; it re-initializes
//   and sets up the scoring engine for the new contest, and tells
//   the track to watch for early starts (cheating).
//   The team ids *must* be set for this to take place!
//
void Track::ButtonResetMsg(RTMessage)
{
  ready = FALSE;

  // complain if the team ids are not set
  if((TrackATeam == 40) || (TrackBTeam == 40))  // Check for Valid Teams
        {
	MessageBox(HWindow, "No Teams are selected!", "Got to Pick Teams!", MB_OK);
	SelectedTeams = FALSE;
        }

  //  if the team ids are set, do the stuff
  if(SelectedTeams)
  	{
	//  clear the scoring engine
	//  set up the scoring engine for the next comp.
	//  wake up the track to watch the beam while waiting for the buzzer

	_Global.SetScore(TrackATeam, 0);
  	_Global.SetScore(TrackBTeam, 0);
  	Scoring.Reset();
  	Scoring.setTeamIds(TrackATeam, TrackBTeam);
  	TrackABmp = 1;
	TrackBBmp = 1;
	Scoring.resetTrack();
        ready = TRUE;
	}

}


////////////////////////////////////////////////////////////////
//  Code executed upon pressing the "Start" button.
//    This is step two in starting the contest.  The team ids
//    must be set *and* the reset button must have been pressed
//    to execute the start functions!
//    If so, this sets up and starts the timer, and
//    tells the ScoringEngine to start the competition. 
//
void Track::ButtonStartMsg(RTMessage)
{
  if((TrackATeam == 40) || (TrackBTeam == 40))  // Check for Valid Teams
        {
	MessageBox(HWindow, "No Teams are selected!", "Got to Pick Teams!", MB_OK);
	SelectedTeams = FALSE;
        }

  if( SelectedTeams && ready )
	{
        // ready the timer
  	newAsec = newBsec = newAmin = newBmin = 0;
  	StartTimer = TRUE;
  	gettime(&t);
  	basesec = t.ti_sec;
  	basemin = t.ti_min;

        // GO
        Scoring.startCompetition(/**/);

  	TimerCountA = TimerCountB = TRUE;
        }

}



///////////////////////////////////////////////
// The Stop Button...  this is the manual stop.
// It calls the same function that is executed
// when the contest ends normally.
//
void Track::ButtonStopMsg(RTMessage)
{
  StoppingFunction();

}



//////////////////////////////////////////////////////////
//  Show a nifty little "About" box on stray mouse clicks
//    
void Track::WMLButtonDown(RTMessage)
{
  MessageBox(HWindow, "IEEE SouthEastCON 1999", "About", MB_OK);

}


///////////////////////////////////////////
//  a nasty windowing function.  let it be.
//
void Track::WMSize( TMessage &Message )
{
  TBaseDemoWindow::WMSize( Message );
  if (IsIconic(HWindow)) {
    if (!Iconized) {
      Iconized = TRUE;   

    };
  } else {
    if (Iconized) {    
      Iconized = FALSE;
    };
  };
}


///////////////////////////////////////////////////////////////////////////
// This is the great big function in which everything that happens happens.
// On every timeout, TimerTick executes; this way we update and display
// all the important information several times each second.
// 
void Track::TimerTick()
{
    char TimerA[10];
    char TimerB[10];
    char D[20];
    char E[20];

    if (IsIconic(HWindow))
    	{
	// why run a competition iconified?
    	}
    else
      	{
	// this is the normal case
	        
        // do windowing stuff
	HDC DC = GetDC(HWindow);
	RECT RC;
        GetClientRect(HWindow, &RC);
	RC.left = RC.left + 200;

	InvalidateRect(HWindow, &RC, FALSE);


        // do the following only when the timer is set to run
    	if((StartTimer) )
	   {
	   gettime(&t);

           //  do this stuff if the team on track A is still competing
	   if (TimerCountA)
		{
		newAsec = (t.ti_sec - basesec);
        	if(newAsec < 0)
			newAsec += 60;

		if(newAsec == 0)
                       	{
			newAmin = (t.ti_min - basemin);
                       	if(newAmin < 0)
                               	newAmin += 60;
			}

                // if they've reached the end, "turn off" their timer
		if (Scoring.isEnd(TRACKA))
                        {
                        TimerCountA = FALSE;
			_Global.SetTime(TrackATeam, (newAmin * 60) + newAsec);
                        }
		}   /* end of if (TimeCountA) */

           // if the team of track B is still competing, do this stuff
	   if (TimerCountB)
               	{
		newBsec = (t.ti_sec - basesec);

		if(newBsec < 0)
			newBsec += 60;

		if((newBsec == 0 ) && (TimerCountB))
                       	{
			newBmin = (t.ti_min - basemin);
                       	if(newBmin < 0)
                       		newBmin += 60;
			}

		// if they've reached the end, "turn off" their timer
		if (Scoring.isEnd(TRACKB))
                        {
                        TimerCountB = FALSE;
			_Global.SetTime(TrackATeam, (newAmin * 60) + newAsec);
			}
		}  /*  end of if (TimerCountB) */


	   Scoring.updateScore();

    	}    /*  end of if (StartTimer) */


        //  if the timer is set to run but both teams have finished, stop.
	if((StartTimer) && (!TimerCountA) && (!TimerCountB))
                {
                StoppingFunction();
		}



        // put together info to be displayed about track A
	sprintf(D, "%3.1f      ", _Global.GetScore(TrackATeam));
	sprintf(TimerA, "%02d:%02d ", newAmin, newAsec);

        // put together info to be displayed about track b
	sprintf(E, "%3.1f      ", _Global.GetScore(TrackBTeam));
	sprintf(TimerB, "%02d:%02d ", newBmin, newBsec);

        // display the text 
	SetTextColor(DC, RGB(255, 255, 255));
    	SetBkColor(DC, RGB(0, 0, 0));
	TextOut(DC, XPlace + 80, 10, TimerA, strlen(TimerA));
	TextOut(DC, XPlace + 80, 180, TimerB, strlen(TimerB));
	TextOut(DC, XPlace + 150, 10, D, strlen(D));
        TextOut(DC, XPlace + 150, 180, E, strlen(E));

	ReleaseDC(HWindow, DC);

        // find out which bitmaps to display next time
	TrackABmp = Scoring.updateBitmap(TRACKA);
	TrackBBmp = Scoring.updateBitmap(TRACKB);


        // check if the teams have been selected...
	if((TrackATeam != 40) && (TrackBTeam != 40))
  	  	SelectedTeams = TRUE;

     	}
  
}  /* end of Track::TimerTick() */
  

//////////////////////////////////////////////////////////////////////
//  what to do when the contest has finished (either both cars reached
//  the end, or someone has pressed "Stop").
//    Stop all the timers, sort the teams by score, clean up the window,
//    and un-set the teams.
//
void Track::StoppingFunction(void)
{
  StartTimer = FALSE;

  TimerCountA = TimerCountB = FALSE;
  _Global.SortScores();
  EraseBackground = TRUE;
  TrackATeam = TrackBTeam = 40;
  _Global.SetScore(40, 0);
  
}




/////////////////////////////////////////////////////
//  lastly, a utility function to draw the bitmaps...
//
void Track::DrawBMP( HDC DC, int X, int Y, HBITMAP BitMap )
{

	HDC MemDC;
	BITMAP bm;
	BOOL MadeDC;

	if ( DC == 0 )
	{
		DC = GetDC( HWindow );
		MadeDC = TRUE;
	}
	else
		MadeDC = FALSE;


	MemDC = CreateCompatibleDC( DC );
	SelectObject( MemDC, BitMap );
	GetObject(Blank, sizeof( bm ), ( LPSTR ) &bm );
	BitBlt( DC, X, Y, bm.bmWidth, bm.bmHeight, MemDC, 0, 0, SRCCOPY );
	DeleteDC( MemDC );
	if ( MadeDC )
		ReleaseDC( HWindow, DC );
}

//  end of track.cpp